<?php

namespace Drupal\entity_bundle_scaffold\Plugin\EntityBundleClassMethodGenerator;

use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\entity_bundle_scaffold\EntityBundleClassMethodGeneratorBase;
use PhpParser\Builder\Method;
use PhpParser\Node\NullableType;

/**
 * A getter method generator for text fields.
 *
 * @EntityBundleClassMethodGenerator(
 *   id = "text",
 *   provider = "core",
 * )
 */
class Text extends EntityBundleClassMethodGeneratorBase {

  /**
   * {@inheritdoc}
   */
  public function buildGetter(FieldDefinitionInterface $field, Method $method, array &$uses): void {
    if ($this->helper->isFieldMultiple($field)) {
      if ($this->helper->supportsArrowFunctions()) {
        $expression = sprintf('return array_map(
                    fn ($item): string => $item->processed,
                    iterator_to_array($this->get(\'%s\'))
                );', $field->getName());
      }
      else {
        $expression = sprintf('return array_map(
                    function ($item): string { return $item->processed; },
                    iterator_to_array($this->get(\'%s\'))
                );', $field->getName());
      }
    }
    else {
      $expression = sprintf('return $this->get(\'%s\')->processed;', $field->getName());
    }

    if ($this->helper->isFieldMultiple($field)) {
      $method->setReturnType('array');
      $method->setDocComment('/** @return string[] */');
    }
    elseif ($field->isRequired()) {
      $method->setReturnType('string');
    }
    elseif ($this->helper->supportsNullableTypes()) {
      $method->setReturnType(new NullableType('string'));
    }
    else {
      $method->setDocComment('/** @return string|null */');
    }

    $method->addStmts($this->helper->parseExpression($expression));
  }

}
